@@ -4,15 +4,18 @@
 #include <set>
 #include <string>
 
-class mixerctl {
-    public:
-	enum channeltype {
-	    ctVolume = 0, ctBass, ctTreble, ctSynth, ctPCM, ctSpeaker, ctLine,
-	    ctMic, ctCD, ctMix, ctPCM2, ctRecord, ctInput, ctOutput, ctLine1,
-	    ctLine2, ctLine3, ctDigital1, ctDigital2, ctDigital3, ctPhoneIn,
-	    ctPhoneOut, ctVideo, ctRadio, ctMonitor, channeltype_end
-	};
+#include "orpheusconf.h"
 
+enum channeltype {
+    ctVolume = 0, ctBass, ctTreble, ctSynth, ctPCM, ctSpeaker, ctLine,
+    ctMic, ctCD, ctMix, ctPCM2, ctRecord, ctInput, ctOutput, ctLine1,
+    ctLine2, ctLine3, ctDigital1, ctDigital2, ctDigital3, ctPhoneIn,
+    ctPhoneOut, ctVideo, ctRadio, ctMonitor, channeltype_end
+};
+
+ENUM_PLUSPLUS(channeltype)
+
+class mixerctl {
     protected:
 	int fd;
 	std::string devname;
