@@ -1,9 +1,9 @@
 /*
 *
 * orpheus text UI class implementation
-* $Id: uitext.cc,v 1.34 2004/02/17 01:18:21 konst Exp $
+* $Id: uitext.cc,v 1.35 2006/05/13 09:14:28 konst Exp $
 *
-* Copyright (C) 2002-4 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2002-6 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "cdtrack.h"
 #include "mp3track.h"
 #include "oggtrack.h"
+#include "streamtrack.h"
 #include "orpheusconf.h"
 #include "mixerctl.h"
 
@@ -235,12 +236,17 @@ void OrpheusTextUI::update() {
     int pos, length, desclen;
     char buf[64];
 
-    desclen = COLS-28;
     trackm.clear();
 
     for(i = plist.begin(); i != plist.end(); i++) {
 	desc = currenttrack == *i ? ">" : " ";
 	desc += (*i)->getdescription();
+	desclen = COLS-28;
+
+	length = (*i)->getlength();
+	if(length == -1)
+	    desclen += 6;
+
 	pos = desclen-7-desc.size();
 
 	if(pos >= 0) {
@@ -250,9 +256,10 @@ void OrpheusTextUI::update() {
 	    desc += ".. ";
 	}
 
-	length = (*i)->getlength();
-	sprintf(buf, " %02d:%02d", (int) length/60, (int) length-(length/60)*60);
-	desc += buf;
+	if(length != -1) {
+	    sprintf(buf, " %02d:%02d", (int) length/60, (int) length-(length/60)*60);
+	    desc += buf;
+	}
 
 	trackm.additem(desc);
     }
@@ -396,6 +403,7 @@ void OrpheusTextUI::showhelp() {
     help.push_back(spair(_("Pause"), "space/p"));
     help.push_back(spair(_("Skip ahead/back"), "}/{ ]/["));
     help.push_back(spair(_("Add/Remove item"),  "a/d"));
+    help.push_back(spair(_("Add broadcast"), "b"));
     help.push_back(spair(_("Next/Prev track"), "+/-"));
     help.push_back(spair(_("Move Up/Down"), "<-/->"));
     help.push_back(spair(_("Edit tags"), "e"));
@@ -422,7 +430,8 @@ void OrpheusTextUI::showhelp() {
 void OrpheusTextUI::addtrackf(const string &fname) {
     string ext = up(fname.substr(fname.size()-4));
     if(ext == ".MP3") plist.push_back(new mp3track(fname)); else
-    if(ext == ".OGG") plist.push_back(new oggtrack(fname));
+    if(ext == ".OGG") plist.push_back(new oggtrack(fname)); else
+    if(fname.find("://") != -1) plist.push_back(new streamtrack(fname));
 }
 
 void OrpheusTextUI::doadd(const string &aitem) {
@@ -490,9 +499,11 @@ void OrpheusTextUI::configuration() {
 
     string mp3player = conf.getmp3player();
     string oggplayer = conf.getoggplayer();
+    string streamplayer = conf.getstreamplayer();
     string cddev = conf.getcddevice();
     string mixerdev = conf.getmixerdevice();
     string proxy = conf.getproxy();
+    string radioxml = conf.getradioxml();
 
     bool autofetch = conf.getautofetch();
     bool russian = conf.getrussian();
@@ -532,6 +543,12 @@ void OrpheusTextUI::configuration() {
 	i = t.addnode(_(" OGG player "));
 	t.addleaff(i, 0, 50, _(" Command line : %s "), oggplayer.c_str());
 
+	i = t.addnode(_(" Radio "));
+	t.addleaff(i, 0, 70, _(" Command line : %s "), streamplayer.c_str());
+#ifdef HAVE_LIBGHTTP
+	t.addleaff(i, 0, 71, _(" Presets URL : %s "), radioxml.c_str());
+#endif
+
 	i = t.addnode(_(" CD player "));
 	t.addleaff(i, 0, 20, _(" Device : %s "), cddev.c_str());
 
@@ -577,14 +594,16 @@ void OrpheusTextUI::configuration() {
 			autosavepl = !autosavepl;
 			break;
 		    case 41:
-			if(++((int) playmode) == PlayMode_size)
+			playmode++;
+			if(playmode == PlayMode_size)
 			    playmode = Normal;
 			break;
 		    case 42:
 			autoplay = !autoplay;
 			break;
 		    case 43:
-			if(++((int) sortorder) == SortOrder_size)
+			sortorder++;
+			if(sortorder == SortOrder_size)
 			    sortorder = byFileName;
 			break;
 		    case 44:
@@ -596,13 +615,21 @@ void OrpheusTextUI::configuration() {
 		    case 60:
 			oggplayer = inputstr(_("Mixer device filename: "), mixerdev);
 			break;
+		    case 70:
+			streamplayer = inputstr(_("Stream player command line: "), streamplayer);
+			break;
+		    case 71:
+			radioxml = inputstr(_("Radio stations presets file: "), radioxml);
+			break;
 		}
 	    } else {
 		conf.setmp3player(mp3player);
 		conf.setoggplayer(oggplayer);
+		conf.setstreamplayer(oggplayer);
 		conf.setcddevice(cddev);
 		conf.setmixerdevice(mixerdev);
 		conf.setproxy(proxy);
+		conf.setradioxml(radioxml);
 		conf.setautofetch(autofetch);
 		conf.setrussian(russian);
 		conf.setautosaveplaylist(autosavepl);
@@ -629,7 +656,7 @@ void OrpheusTextUI::lookfor(string sub) {
 
     while(nt != plist.end()) {
 	if(up((*nt)->getdescription()).find(sub) != -1) {
-	string a = (*nt)->getdescription();
+	    string a = (*nt)->getdescription();
 	    trackm.setpos(nt-plist.begin());
 	    update();
 	    break;
@@ -642,13 +669,13 @@ void OrpheusTextUI::mixer() {
     dialogbox db;
     int n, b, i, width, val, len;
     bool finished = false;
-    mixerctl::channeltype ct;
+    channeltype ct;
     string bar;
 
-    static map<mixerctl::channeltype, string> cnames;
+    static map<channeltype, string> cnames;
     static int maxtlen = 0;
 
-    set<mixerctl::channeltype> chavail;
+    set<channeltype> chavail;
     mixerctl mix(conf.getmixerdevice());
 
     if(!mix.is_open()) {
@@ -657,34 +684,34 @@ void OrpheusTextUI::mixer() {
     }
 
     if(cnames.empty()) {
-	cnames[mixerctl::ctVolume] = _("Volume");
-	cnames[mixerctl::ctBass] = _("Bass");
-	cnames[mixerctl::ctTreble] = _("Treble");
-	cnames[mixerctl::ctSynth] = _("Synth");
-	cnames[mixerctl::ctPCM] = _("PCM");
-	cnames[mixerctl::ctSpeaker] = _("Speaker");
-	cnames[mixerctl::ctLine] = _("Line");
-	cnames[mixerctl::ctMic] = _("Microphone");
-	cnames[mixerctl::ctCD] = _("CD");
-	cnames[mixerctl::ctMix] = _("Mix");
-	cnames[mixerctl::ctPCM2] = _("PCM 2");
-	cnames[mixerctl::ctRecord] = _("Record");
-	cnames[mixerctl::ctInput] = _("Input");
-	cnames[mixerctl::ctOutput] = _("Output");
-	cnames[mixerctl::ctLine1] = _("Line 1");
-	cnames[mixerctl::ctLine2] = _("Line 2");
-	cnames[mixerctl::ctLine3] = _("Line 3");
-	cnames[mixerctl::ctDigital1] = _("Digital 1");
-	cnames[mixerctl::ctDigital2] = _("Digital 2");
-	cnames[mixerctl::ctDigital3] = _("Digital 3");
-	cnames[mixerctl::ctPhoneIn] = _("Phone In");
-	cnames[mixerctl::ctPhoneOut] = _("Phone Out");
-	cnames[mixerctl::ctVideo] = _("Video");
-	cnames[mixerctl::ctRadio] = _("Radio");
-	cnames[mixerctl::ctMonitor] = _("Monitor");
+	cnames[ctVolume] = _("Volume");
+	cnames[ctBass] = _("Bass");
+	cnames[ctTreble] = _("Treble");
+	cnames[ctSynth] = _("Synth");
+	cnames[ctPCM] = _("PCM");
+	cnames[ctSpeaker] = _("Speaker");
+	cnames[ctLine] = _("Line");
+	cnames[ctMic] = _("Microphone");
+	cnames[ctCD] = _("CD");
+	cnames[ctMix] = _("Mix");
+	cnames[ctPCM2] = _("PCM 2");
+	cnames[ctRecord] = _("Record");
+	cnames[ctInput] = _("Input");
+	cnames[ctOutput] = _("Output");
+	cnames[ctLine1] = _("Line 1");
+	cnames[ctLine2] = _("Line 2");
+	cnames[ctLine3] = _("Line 3");
+	cnames[ctDigital1] = _("Digital 1");
+	cnames[ctDigital2] = _("Digital 2");
+	cnames[ctDigital3] = _("Digital 3");
+	cnames[ctPhoneIn] = _("Phone In");
+	cnames[ctPhoneOut] = _("Phone Out");
+	cnames[ctVideo] = _("Video");
+	cnames[ctRadio] = _("Radio");
+	cnames[ctMonitor] = _("Monitor");
 
 	maxtlen = 0;
-	map<mixerctl::channeltype, string>::const_iterator ict = cnames.begin();
+	map<channeltype, string>::const_iterator ict = cnames.begin();
 	while(ict != cnames.end()) {
 	    if(ict->second.size() > maxtlen) maxtlen = ict->second.size();
 	    ++ict;
@@ -719,7 +746,7 @@ void OrpheusTextUI::mixer() {
     while(!finished) {
 	m.clear();
 
-	for(ct = mixerctl::ctVolume; ct != mixerctl::channeltype_end; (int) ct += 1) {
+	for(ct = ctVolume; ct != channeltype_end; ct++) {
 	    if(chavail.count(ct)) {
 		val = mix.readlevel(ct);
 		if(val >= 0) {
@@ -734,7 +761,7 @@ void OrpheusTextUI::mixer() {
 	finished = !db.open(n, b, (void **) &i);
 
 	if(!finished) {
-	    ct = (mixerctl::channeltype) i;
+	    ct = (channeltype) i;
 	    val = mix.readlevel(ct);
 
 	    switch(b) {
@@ -809,6 +836,10 @@ int OrpheusTextUI::tracklistkeys(verticalmenu &m, int k) {
 	    cdtrack::readtracks();
 	    break;
 
+	case 'b':
+	    tui.importRadio();
+	    break;
+
 	case 'j':
 	case 'J':
 	    if(tui.currenttrack)
@@ -1183,6 +1214,119 @@ bool OrpheusTextUI::recentlists() {
     return false;
 }
 
+void OrpheusTextUI::importRadio() {
+    vector<streamtrack::TopGroup> psets = streamtrack::loadPresets();
+    set<int> opennodes, marked;
+
+    dialogbox db;
+    int n, b, i, id;
+    bool finished = false, added = false;
+    string url;
+
+    textwindow w(0, 0, (int) (COLS*0.85), (int) (LINES*0.8),
+	schemer[DialogFrames], TW_CENTERED);
+
+    w.set_title(schemer[DialogHighLight], _(" Select radio broadcasts "));
+
+    db.setwindow(&w, false);
+
+    db.settree(new treeview(schemer[DialogText],
+	schemer[DialogSelected], schemer[DialogHighLight],
+	schemer[DialogText]));
+
+    db.setbar(new horizontalbar(schemer[DialogText],
+	schemer[DialogSelected], _("Add URL"), _("Mark/Unmark"), _("Done"), 0));
+
+    db.idle = &dialogidle;
+    db.addautokeys();
+
+    db.addkey(KEY_IC, 1);
+    db.addkey(' ', 1);
+
+    db.getbar()->item = 1;
+    treeview &t = *db.gettree();
+    t.collapsable = true;
+
+    while(!finished) {
+	if(!t.empty()) {
+	    opennodes.clear();
+
+	    for(i = 0; i < t.menu.getcount(); i++) {
+		int id = t.getid(i);
+
+		if(t.isnode(id))
+		if(t.isnodeopen(id))
+		    opennodes.insert(id);
+	    }
+	}
+
+	t.clear();
+
+	for(vector<streamtrack::TopGroup>::const_iterator it = psets.begin(); it != psets.end(); ++it) {
+	    int ntopgroup = t.addnodef(" %s ", it->name.c_str());
+	    if(opennodes.count(ntopgroup)) t.opennode(ntopgroup);
+
+	    for(vector<streamtrack::SubGroup>::const_iterator is = it->subgroups.begin(); is != it->subgroups.end(); ++is) {
+		int nsubgroup = t.addnodef(ntopgroup, 0, 0, " %s ", is->name.c_str());
+		if(opennodes.count(nsubgroup)) t.opennode(nsubgroup);
+
+		for(vector<streamtrack::Station>::const_iterator ist = is->stations.begin(); ist != is->stations.end(); ++ist) {
+		    id = nsubgroup+ist-is->stations.begin()+1;
+
+		    t.addleaff(nsubgroup,
+			marked.count(id) ? schemer[DialogFrames] : 0,
+			(void *) &(*ist), " %s ", ist->name.c_str());
+		}
+	    }
+	}
+
+	finished = !db.open(n, b, (void **) &i);
+
+	if(!finished) {
+	    switch(b) {
+		case 0:
+		    url = inputstr(_("Broadcast URL: "));
+		    if(url.find("://") != -1) {
+			plist.push_back(new streamtrack(url));
+			finished = added = true;
+		    }
+		    break;
+
+		case 1:
+		    id = t.getid(n-1);
+
+		    if(!t.isnode(id)) {
+			if(marked.count(id)) marked.erase(id);
+			    else marked.insert(id);
+			t.setcur(t.getid(n));
+		    } else {
+			if(t.isnodeopen(id)) t.closenode(id);
+			    else t.opennode(id);
+		    }
+		    break;
+
+		case 2:
+		    if(!marked.empty()) {
+			for(set<int>::const_iterator im = marked.begin(); im != marked.end(); ++im) {
+			    streamtrack::Station *st = (streamtrack::Station *) t.getref(*im);
+			    if(!st->urls.empty()) {
+				streamtrack *t = new streamtrack(st->urls.front());
+				t->station = st->name;
+				plist.push_back(t);
+			    }
+			}
+
+			finished = added = true;
+		    }
+		    break;
+	    }
+	}
+    }
+
+    db.close();
+    if(added) update();
+}
+
 void OrpheusTextUI::usage(const string &argv0) {
     cout << _("Usage: ") << argv0 << " [OPTION].." << endl << endl;
 
